import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from scipy.integrate import trapz

# Read the CO2 concentration values from the file
co2_ppm_file_path = "co2_ppm.txt"
try:
    co2_ppm_values = pd.read_csv(co2_ppm_file_path, header=None, names=['CO2_ppm'])
except FileNotFoundError:
    print(f"Error: File '{co2_ppm_file_path}' not found.")
    exit()
except pd.errors.EmptyDataError:
    print(f"Error: File '{co2_ppm_file_path}' is empty.")
    exit()

# Read the data from the CO2 bending mode file
co2_bend_file_path = "co2_bend.txt"
try:
    data = pd.read_csv(co2_bend_file_path, header=None, names=['Wavenumber', 'AbsorptionIntensity'])
except FileNotFoundError:
    print(f"Error: File '{co2_bend_file_path}' not found.")
    exit()
except pd.errors.EmptyDataError:
    print(f"Error: File '{co2_bend_file_path}' is empty.")
    exit()

# Initialize lists to store results for each CO2 concentration
trans_values = []
temp_surf_values = []

# Fixed temperature
temp = 288

# Loop through each CO2 concentration value
for co2_ppm in co2_ppm_values['CO2_ppm']:
    # Normalize CO2 concentration to ppm
    co2_ppm /= 1e6
    
    # Calculate cross-section number density
    xsec_num_density = 234972 * co2_ppm

    # Multiply each absorption intensity by 96.8 to obtain relative absorbance (Arel)
    data['RelativeAbsorbance'] = data['AbsorptionIntensity'] * xsec_num_density

    # Convert relative absorbance to flux transmittance using the provided formula
    data['FluxTransmittance'] = (np.exp(-(data['RelativeAbsorbance'] * 2.245)) + np.exp(-(data['RelativeAbsorbance'] * 1.13))) / 2

    # Generate wavenumbers from 0.01 to 2399.99 cm^-1
    wvn = np.arange(0.01, 2400, 0.01)

    # Calculate Planck distribution for the fixed temperature
    planck_distribution = (0.000000037211 * wvn**3) / (np.exp(1.435 * wvn / temp) - 1)

    # Multiply flux transmittance by Planck distribution at each wavenumber
    data['Result'] = data['FluxTransmittance'] * np.interp(data['Wavenumber'], wvn, planck_distribution)

    # Calculate the cumulative sum for Planck distribution and CO2 Transmittance
    cumulative_sum_planck = np.cumsum(planck_distribution)
    cumulative_sum_co2_trans = np.cumsum(data['Result'])

    # Convert co2_ppm to a string
    co2_ppm_str = str(co2_ppm)

    # Create the file name
    file_name = f"planck_radiation_{co2_ppm_str}_ppm_int.csv"

    print(file_name)

    # Save results to CSV files
    result_df = pd.DataFrame({'Wavenumber': data['Wavenumber'], 'CO2 Planck': cumulative_sum_co2_trans})
    result_df.to_csv('co2_ppm_trans_to_planck.csv', index=False)
    print("Saved results to 'planck_radiation_CO2_ppm_int.csv'.")

    result_df_planck = pd.DataFrame({'Wavenumber': data['Wavenumber'], 'Planck': cumulative_sum_planck})
    result_df_planck.to_csv(file_name, index=False)
    print("Saved results to",file_name, ".")

    # Plot the cumulative sum of Planck distribution and CO2 Transmittance
    plt.figure(figsize=(10, 6))
    plt.plot(data['Wavenumber'], cumulative_sum_planck, label='Cumulative Sum Planck')
    plt.plot(data['Wavenumber'], cumulative_sum_co2_trans, label='Cumulative Sum CO2 Transmittance')

    plt.xlabel('Wavenumber')
    plt.ylabel('Cumulative Sum')
    plt.title('Cumulative Sum of Planck Distribution and CO2 Transmittance')
    plt.legend()

    # Add legend for CO2 concentration
    plt.annotate(f'CO2 ppm: {co2_ppm}', xy=(0.7, 0.1), xycoords='axes fraction', fontsize=10, color='gray')
    plt.show()